#First, the intercept-only model was estimated; the intercept denotes the average difference effect size between original and replication study. Second, to test for small study effects, we added the standard error of the
#Reproducibility Project original study as a predictor, akin to Egger’s test; a positive effect is often interpreted as evidence for publication bias. Our third model tested the effect of discipline.


library(here)
library(tidyverse)
library(stringi)
library(metafor)
## Load data into R:
## ----------------
RP_data <- read.csv(here("RP_data", "data.csv"))


## Merge RP_data with ReplicationSuccess::RProjects:
## -------------------------------------------------

# Filter out the relevant projects from the RP_data and extract the name of the
# first and second author of the original study:
RP_data <- RP_data %>%
  filter(project %in% c("ee", "rpp")) %>%
  filter(!is.na(effect_size.o),
         !is.na(effect_size.r)) %>%
  mutate(first_author = stri_trans_general(paste0(
    ifelse(grepl("et al", authors.o),
           str_split(authors.o, "et al", simplify = TRUE)[, 1],
           str_split(authors.o, " and", simplify = TRUE)[, 1]),
    " "), id = "Latin-ASCII"),
    second_author = stri_trans_general(ifelse(grepl("et al", authors.o), "et al",
                                              str_split(authors.o, " and ", simplify = TRUE)[, 2]),
                                       id = "Latin-ASCII"))

# Filter out the relevant projects from ReplicationSuccess::RProjects and
# extract the last name of the first and second author of the original study:
RProjects_psychology_econ <- ReplicationSuccess::RProjects %>%
  filter(project %in% c("Experimental Economics", "Psychology")) %>%
  mutate(first_author =
           stri_trans_general(
             paste0(
               ifelse(project == "Experimental Economics",
                      ifelse(grepl("et al", study),
                             str_split(study, " et al", simplify = TRUE)[, 1],
                             str_split(study, " and", simplify = TRUE)[, 1]),
                      word(str_split(study, ",", simplify = TRUE)[, 1],
                           2, -1)), " "), id = "Latin-ASCII"),
         second_author =
           stri_trans_general(
             ifelse(project == "Psychology",
                    str_split(study, ", ", simplify = TRUE)[, 2],
                    ifelse(grepl("et al", study), "etal",
                           str_split(study, " and ", simplify = TRUE)[, 2])),
             id = "Latin-ASCII"),
         second_author = ifelse(project == "Psychology",
                                word(second_author, 2, -1),
                                word(second_author, 1, -3)))

# Now, let's check whether the first author shows up as first author in the
# RP_data and collect the ids
ids <-
  sapply(unique(RProjects_psychology_econ$first_author),
         function(name) which(grepl(tolower(name),
                                    tolower(RP_data$first_author))))

# Some first authors show up more than once. let's first deal with the studies
# for which a unique id was found:

## 1. Unique ID
unique <- unlist(ids[which(sapply(ids, function (id) length(id)) == 1)])
# Create the id_in_data variable as the covariate to merge datasets
RProjects_psychology_econ$id_in_data <- NA
# Where unique ID, set the covariate to that ID
RProjects_psychology_econ[
  match(names(unique),
        RProjects_psychology_econ$first_author), "id_in_data"] <- unique

## 2. Double IDs (either first author more than one study included or the same
# study was replicated twice)
where_double <- ids[which(sapply(ids, function (id) length(id)) > 1)]

# 2.a. "first author more than one study" (most likely different second author)
# Get second author
second_RP_data <- RP_data[unlist(where_double), "second_author"]
names(second_RP_data) <- unlist(where_double)
second_author_RProjects <- RProjects_psychology_econ %>%
  filter(first_author %in% names(where_double)) %>%
  pull(second_author)

ids_second <- sapply(second_author_RProjects,
                     function(name) which(grepl(name, second_RP_data)))
unique <- unlist(ids_second[which(sapply(ids_second, function (id) length(id)) == 1)])
ids_bis <- names(second_RP_data)[unique]
names(ids_bis) <- names(unique)

RProjects_psychology_econ <- RProjects_psychology_econ %>%
  mutate(id_in_data =
           ifelse(first_author %in% names(where_double) & second_author %in% second_author_RProjects,
                  ids_bis[second_author], id_in_data))

# 2.b. "study replicated more than once"
where_double_second_bis <- ids_second[which(sapply(ids_second, function (id) length(id)) > 1)]
where_double_second <- unique(where_double_second_bis)
names(where_double_second) <- RProjects_psychology_econ %>%
  filter(second_author %in% unique(names(where_double_second_bis))) %>%
  pull(first_author) %>% unique()

# The replication sample size will be used to identify the correct study pairs:
n_r_RProjects_psychology_econ <- RProjects_psychology_econ %>%
  filter(first_author %in% names(where_double_second)) %>%
  pull(nr)

# Find them in the RP_data:
ids_n <- unlist(where_double[names(where_double_second)])[
  sapply(n_r_RProjects_psychology_econ,
         function(nr)
           which(abs(nr -
                       RP_data[unlist(where_double[names(where_double_second)]),
                               "n.r"]) < 5))]
names(ids_n) <- paste0(names(where_double_second_bis),
                       n_r_RProjects_psychology_econ)


RProjects_psychology_econ <-
  RProjects_psychology_econ %>%
  mutate(id_in_data =
           ifelse(is.na(id_in_data) &
                    nr %in% n_r_RProjects_psychology_econ,
                  ids_n[paste0(second_author, nr)], id_in_data))

# MERGE with id_in_data
RP_data_merged <- RProjects_psychology_econ %>%
  left_join(RP_data %>%
              mutate(row_id = as.character(1:n())) %>%
              dplyr::select(authors.o, pub_year, journal, discipline, length, citations,
                            n.r,
                            power.o:us_lab.r, row_id),
            by = c("id_in_data" = "row_id"))

# Check if merge worked:
# RP_data_merged %>% select(study, authors.o, everything()) %>% view()

# A couple of original studies in the ReplicationSuccess::RProjects were not
# present in the RP_data.
# length(which(is.na(RProjects_psychology_econ$id_in_data)))
# --> those were non-significant original studies.

# To be deleted
RP_data_merged <- RP_data_merged %>%
  filter(!is.na(id_in_data)) %>%
  dplyr::select(-first_author,
                -second_author,
                -id_in_data)

# ReplicationSuccess::RProjects %>% filter(project == "Psychology") %>% filter(po>0.05)

# Problem: ReplicationSuccess::RProjects %>%
#                     filter(study == "P Bressan, D Stranieri")
# has two replications: keep only the one that is NOT online
# (replication report: https://osf.io/7vriw)
# To be discussed somewhere in paper or supplement
RP_data_merged <- RP_data_merged %>%
  filter(!((study == "P Bressan, D Stranieri") & (online.r == 0)))


## Data quality checks:
## --------------------
# According to the supplement seniority.r == Assistant does not exist, so it
# must be an error and should be Assistant Professor instead

RP_data_merged <- RP_data_merged %>%
  mutate(seniority.r = ifelse(seniority.r == "Assistant",
                              "Assistant Professor", seniority.r))


## Complexity reduction of factor variables:
## ----------------------------------------
RP_data_merged <- RP_data_merged %>%
  mutate(seniority.r = ifelse(seniority.r %in% c("Assistant Professor",
                                                 "Associate Professor"),
                              "Assistant/ociate Pr.", seniority.r),
         seniority.o = ifelse(seniority.o %in% c("Assistant Professor",
                                                 "Associate Professor"),
                              "Assistant/ociate Pr.", seniority.o),
         compensation.o = ifelse(compensation.o %in% c("credit", "mixed"),
                                 "credit or mixed", compensation.o),
         compensation.r = ifelse(compensation.r %in% c("credit", "mixed"),
                                 "credit or mixed", compensation.r))

#####

# The dependent variable was the difference of Fisher-transformed correlations
# (original – replication)
# Add a column with the differences in effect size, and their standard errors
RP_data_merged <- RP_data_merged %>%
  mutate(diff = fiso - fisr,
         se_diff = sqrt(se_fiso**2 + se_fisr**2))

RP_data_merged_RPP <- RP_data_merged %>%
  filter(discipline != "Economics")

# The null-model without predictors yielded an average estimated difference in
# effect size equal to .21 (z = 7.55, p < .001) in favor of the original study.
# The null-hypothesis of homogeneous difference in effect sizes was rejected
# (Q(72) = 152.39, p < .001), with medium observed heterogeneity (𝜏𝜏̂=.149,
# I2 = 47.8%).

null_model <- rma(yi = diff, sei = se_diff, method = "REML",
                  data = RP_data_merged_RPP)
summary(null_model)
# Q(df = 68) = 112.3083, p-val = 0.0006
sqrt(null_model$tau2)
# 0.1403347
summary(null_model)$I2
# 40.73684

# Via Egger’s test, precision of the original study was associated with the
# difference in effect size (b = .85, z = 1.88, one-tailed p = .030), hence
# imprecise original studies (large standard error) yielded larger differences
# in effect size between original and replication study. This is confirmed by
# the funnel plot in Figure S7.

egger_model <- rma(yi = diff, sei = se_diff, mods = ~ se_fiso, method = "REML",
                   data = RP_data_merged_RPP)
# summary(egger_model)
other_model <- rma(yi = diff, sei = se_diff, mods = ~ (se_fiso/fiso)**2, method = "REML",
                   data = RP_data_merged_RPP)

egger_model <- rma(yi = diff, sei = se_diff, mods = ~ se_fiso, method = "REML",
                   data = RP_data_merged)
# summary(egger_model)
# RP_data_merged <- RP_data_merged %>%
#   mutate(var = (se_fiso/fiso)^2)
# other_model <- rma(yi = diff, sei = se_diff, mods = ~ var, method = "REML",
#                    data = RP_data_merged)

# Discipline was not associated with the difference in effect size, χ2(3) =
# 2.451, p = .48, (i.e., the average difference in effect size was equal for
# JPSP, JEP:LMC, PSCI-soc, and PSCI-cog)
discipline_model <- rma(yi = diff, sei = se_diff, mods = ~ discipline,
                        method = "REML", data = RP_data_merged_RPP)
summary(discipline_model)


# Also, after controlling for the effect of the standard error of the original
# study, no differences between disciplines were observed (𝜒𝜒2(3) = 2.807,
# p = .42). No moderating effects were observed for: importance of the effect
# (b = -.010, p = .77), surprising effect (b = .001, p = .97), experience and
# expertise of original team (b = -.0017, p = .96), challenge of conducting
# replication (b = 0.026, p = .45), and self-assessed quality of replication
# (b = -.037, p = .51). However, a positive effect of experience and expertise
# of replication team was observed (b = .13, p = .0063), meaning that the
# difference between original and replication effect size was higher for
# replication teams with more experience and expertise.


# The results from the three meta-analyses tentatively suggest that the
# journals/disciplines are similarly influenced by publication bias leading to
# overestimated effect sizes, and that cognitive effects are larger than social
# effects on average -- possibly because of the target of study or the
# sensitivity of the research designs (e.g., within-subject designs reducing
# error and increasing sensitivity).