
# Function to get muliplicative or additive heterogeneity
get_heterogeneity <-
  function(theta_o, theta_r, se_o, se_r, multiplicative = TRUE,
           covariate = NULL, export_model = FALSE) {

    # Calculate the difference between RWE (r) and RCT (o)
    mydiff <- theta_r - theta_o
    # ... as well as the weights as the inverse of the variance of the
    # difference
    myweight <- 1/(se_o^2 + se_r^2)
    # Write data frame with response and all covariates:
    if (!is.null(covariate)) mydata <- data.frame(mydiff, covariate)


    # Models
    if (multiplicative){
      # For multiplicative we'll use a simple weighted lm
      # If no covariate is given we add an intercept
      if (is.null(covariate)) {
        model <- lm(mydiff ~ 1, weights = myweight)
      } else {
        # if covariates given, use all the columns in mydata:
        model <- lm(mydiff ~ ., weights = myweight, data = mydata)
      }
      result <- list(bias = model$coefficients[1],
                     heterogeneity_sd = summary(model)$sigma,
                     method = "multiplicative")
      if (export_model) {
        result[["model"]] <- model
      }
      # Return the results:
      return(result)
    } else {
      # For the additive version of the heterogeneity, we apply a mixed model
      # First, we'll define an indicator for the trials. Each row stands for a
      # separate trial. So we can just enumerate the rows aka trials.
      trial <- as.character(seq_along(mydiff))
      if (is.null(covariate)) {
        # If no covariate is given we only add an intercept
        model <- lme(mydiff ~ 1, random = ~ 1 | trial,
                     weights = varFixed(~1/myweight),
                     control = lmeControl(sigma = 1))
      } else {
        form <- paste0("mydiff ~ ",
                       paste0(colnames(mydata)[-1], collapse = " + "))
        mydata$trial <- trial
        mydata$myweight <- myweight
        model <- lme(as.formula(form), random = ~ 1 | trial,
                     data = mydata,
                     weights = varFixed(~1/myweight),
                     control = lmeControl(sigma = 1))
      }

      result <- list(bias = fixef(model)[1],
                     heterogeneity_sd = sqrt(model$modelStruct$reStruct$trial[1]),
                     method = "additive")
      if (export_model) {
        result[["model"]] <- model
      }
      # Return the results
      return(result)
    }
  }


pLimit <- function(z1, z2){
    geomMean <- function(x, y) return(sqrt(x*y))
    W <- geomMean(z1^2, z2^2)
    target <- function(t, W)
        exp(-W/sqrt(t))/sqrt(t*(1-t))
    res <- integrate(target, lower=0, upper=1, W=W)$value
    return(res/pi)
}

PLimit <- Vectorize(pLimit)

pLimitN <- function(z){
    geomMean <- function(x) return(exp(mean(log(x))))
    W <- geomMean(z^2)
    target <- function(t, W)
        exp(-W/sqrt(t))/sqrt(t*(1-t))
    res <- integrate(target, lower=0, upper=1, W=W)$value
    return(res/pi)
}

PLimitN <- Vectorize(pLimitN)


StoufferZ <- function(p, w=rep(1, length(p)), alternative="one.sided"){
    se <- 1/sqrt(w)
    n <- length(p)
    t <- p2z(p, alternative=alternative)
    z <- weighted.mean(t*se, w)*sqrt(sum(w))
    res <- pnorm(z, lower.tail=FALSE)
    if(alternative=="two.sided")
        res <- 2*res
    return(res)
}

combineFisher <- function(p){
    n <- length(p)
    t <- -2*sum(log(p))
    res <- pchisq(t, df=2*n, lower.tail=FALSE)
    return(res)
}

sqrtF <- function(z){
    p <- z2p(z, alternative="one.sided")
    res <- combineFisher(p)
    return(sqrt(res))
}

sqrtZ <- function(z){
    p <- z2p(z, alternative="one.sided")
    res <- StoufferZ(p)
    return(sqrt(res))
}



###############################
## cdf of the random variable Z
## parameter c >= 0
###############################
FZ <- function(z, c){
    stopifnot((c>=0))
    if(z<=0)
        return(0)
    if(c==0)
        return(1-4*(1-pnorm(sqrt(z)))^2)
    if(c==1)
        return(pgamma(z, 1/2, 2))
    f <- function(t, c, z){
##        if(c<1)
##            t1 <- exp(-(1-c)*z/(1-sqrt(1-(1-c)*t)))
##        if(c>1)
        t1 <- exp(-(c-1)*z/(sqrt(1+(c-1)*t)-1))
        t2 <- 1/sqrt(t*(1-t))
        return(t1*t2)
    }
    myint <- integrate(f, lower=0, upper=1, z=z, c=c)$value
    result <- 1-myint/pi
    return(result)
}

######################################################
## complementary cdf of Z
## originally needed to compute expecation numerically
######################################################
cFZ <- function(z, c)
    return(1-FZ(z, c))

EZ <- function(c){
    if(c==1)
        myint <- 0.25
    if(c==0)
        myint <- 1-2/pi
    target <- function(t, c){
        if(c<1)
            term1 <- (1-sqrt(1-(1-c)*t))/(1-c)
        if(c>1)
            term1 <- (sqrt(1+(c-1)*t)-1)/(c-1)
        term2 <- 1/sqrt(t*(1-t))
        term3 <- 1/pi
        return(term1*term2*term3)
    }
    if((c!=0)&(c!=1))
        myint <- integrate(target, lower=0, upper=1, c=c)$value
    return(myint)
}


## computes the p-value for a given c
pValue <- function(z1, z2, c,
                   alternative = c("greater", "less", "two.sided", "none"),
                   bound = FALSE){
    z <- c(z1, z2)
    check_greater <- min(z) >= 0
    check_less <- max(z) <= 0
    n <- 2
    alternative <- match.arg(alternative)
    break_p <- 1/(2^n)

    if(c==1){
        res <- hMeanChiSq(z=c(z1,z2), alternative=alternative)
        return(res)
    }
    if(c==Inf){
        res <- pLimit(z1, z2)
        if(alternative=="one.sided")
            res <- res/4
        return(res)
    }
    if(c!=1 & c < Inf){
        z <- zSceptical(zo=z1, zr=z2, c=c)
        res <- (1-FZ(z^2, c=c))
        if (alternative == "greater") {
            if (bound)
                res <- if (check_greater)
                           res/(2^n)
                       else paste(">", format(break_p, scientific = FALSE))
            else res <- if (check_greater)
                            res/(2^n)
                        else 1 - 2 * sqrt(res/(2^n)) + res/(2^n) ## NaN
        }
        if (alternative == "less") {
            if (bound)
                res <- if (check_less)
                           res/(2^n)
                       else paste(">", format(break_p, scientific = FALSE))
            else res <- if (check_less)
                            res/(2^n)
                        else 1 - 2 * sqrt(res/(2^n)) + res/(2^n) ## NaN
        }
        if (alternative == "two.sided") {
            if (bound)
                res <- if (check_greater || check_less)
                           res/(2^(n - 1))
                       else paste(">", format(2 * break_p, scientific = FALSE))
            else res <- if (check_greater || check_less)
                            res/(2^(n - 1))
                        else 1 - res/(2^(n - 1)) ## NaN
        }
    }
    return(res)
}

pValueMu <- function(thetahat, se, mu = 0, c = 1,
    alternative = c("greater", "less", "two.sided", "none"),
    bound = FALSE)
{
    stopifnot(is.numeric(thetahat), length(thetahat) > 0, is.finite(thetahat),
        is.numeric(se), length(se) == 1 || length(se) == length(thetahat),
        is.finite(se), min(se) > 0, c >= 0, is.numeric(mu),
        length(mu) > 0, is.finite(mu), !is.null(alternative))
    alternative <- match.arg(alternative)
    stopifnot(is.logical(bound), length(bound) == 1, is.finite(bound))
    n <- length(thetahat)
    stopifnot((length(thetahat)==2) & (length(se)==2))
    m <- length(mu)
    z <- (thetahat - mu)/se
    if(c==Inf)
        res <- PLimit(z[1], z[2])
    else{
        zS <- zSceptical(zo=z[1], zr=z[2], c=c)
        res <- (1-FZ(zS^2, c=c))
    }
    check_greater <- min(z) >= 0
    check_less <- max(z) <= 0
    break_p <- 1/(2^n)
    if (alternative == "greater") {
        if (bound)
            res <- if (check_greater)
                       res/(2^n)
                   else paste(">", format(break_p, scientific = FALSE))
        else res <- if (check_greater)
                        res/(2^n)
                    else 1 - 2 * sqrt(res/(2^n)) + res/(2^n) ## NaN
    }
    if (alternative == "less") {
        if (bound)
            res <- if (check_less)
                       res/(2^n)
                   else paste(">", format(break_p, scientific = FALSE))
        else res <- if (check_less)
                        res/(2^n)
                    else 1 - 2 * sqrt(res/(2^n)) + res/(2^n) ## NaN
    }
    if (alternative == "two.sided") {
        if (bound)
            res <- if (check_greater || check_less)
                       res/(2^(n - 1))
                   else paste(">", format(2 * break_p, scientific = FALSE))
        else res <- if (check_greater || check_less)
                        res/(2^(n - 1))
                    else 1 - res/(2^(n - 1)) ## NaN
    }
    return(res)
}

scepticalCI <- function(thetahat, se,
                        alternative = c("two.sided", "greater", "less", "none"),
                        c, level = 0.95) {
    stopifnot(is.numeric(thetahat), length(thetahat) > 0, is.finite(thetahat))
    stopifnot(is.numeric(se), length(se) == 1 || length(se) ==
                              length(thetahat), is.finite(se), min(se) > 0,
              !is.null(alternative))
    alternative <- match.arg(alternative)
    stopifnot(is.numeric(level), length(level) == 1, is.finite(level),
        0 < level, level < 1)
    target <- function(limit) {
        pValueMu(thetahat = thetahat, se = se, c = c, mu = limit,
                 alternative = alternative, bound = FALSE) - alpha
    }
    indOrd <- order(thetahat)
    thetahat <- thetahat[indOrd]
    se <- se[indOrd]
    thetahatUnique <- unique(thetahat)
    nThetahatUnique <- length(thetahatUnique)
    mini <- which.min(thetahat)
    maxi <- which.max(thetahat)
    mint <- thetahat[mini]
    maxt <- thetahat[maxi]
    minse <- se[mini]
    maxse <- se[maxi]
    alpha <- 1 - level
    z1 <- max(-qnorm(alpha), 1)
    eps <- 1e-06
    factor <- 5
    if (alternative == "none") {
        lower <- mint - z1 * minse
        while (target(lower) > 0) lower <- lower - minse
        CIlower <- uniroot(f = target, lower = lower, upper = thetahat[1])$root
        CImiddle <- matrix(NA, nrow = 2, ncol = nThetahatUnique -
            1)
        gam <- matrix(NA, nrow = nThetahatUnique - 1, ncol = 2)
        colnames(gam) <- c("minimum", "pvalue_fun/gamma")
        for (i in 1:(nThetahatUnique - 1)) {
            opt <- optimize(f = target, lower = thetahatUnique[i],
                upper = thetahatUnique[i + 1])
            gam[i, ] <- c(opt$minimum, opt$objective + alpha)
            if (opt$objective <= 0) {
                CImiddle[1, i] <- uniroot(f = target, lower = thetahatUnique[i],
                  upper = opt$minimum)$root
                CImiddle[2, i] <- uniroot(f = target, lower = opt$minimum,
                  upper = thetahatUnique[i + 1])$root
            }
        }
        CImiddle <- CImiddle[!is.na(CImiddle)]
        upper <- maxt + maxse
        while (target(upper) > 0) upper <- upper + z1 * maxse
        CIupper <- uniroot(f = target, lower = thetahat[length(thetahat)],
            upper = upper)$root
        CI <- matrix(c(CIlower, CImiddle, CIupper), ncol = 2,
            byrow = TRUE)
        colnames(CI) <- c("lower", "upper")
        return(list(CI = CI, gamma = gam))
    }
    if (alternative == "two.sided") {
        lower <- uniroot(f = target, lower = mint - factor *
            z1 * minse, upper = mint - eps * minse)$root
        upper <- uniroot(f = target, lower = maxt + eps * maxse,
            upper = maxt + factor * z1 * maxse)$root
        return(list(CI = cbind(lower, upper)))
    }
    if (alternative == "greater") {
        lower <- uniroot(f = target, lower = mint - factor *
            z1 * minse, upper = mint - eps * minse)$root
        upper <- Inf
        return(list(CI = cbind(lower, upper)))
    }
    if (alternative == "less") {
        lower <- -Inf
        upper <- uniroot(f = target, lower = maxt + eps * maxse,
            upper = maxt + factor * z1 * maxse)$root
        return(list(CI = cbind(lower, upper)))
    }
    stop("function not get here.")
}

scepticalCI2 <- function(thetahat, se, alternative = c("two.sided",
                                                   "greater", "less", "none"), c, level = 0.95)
{
  stopifnot(is.numeric(thetahat), length(thetahat) > 0, is.finite(thetahat))
  stopifnot(is.numeric(se), length(se) == 1 || length(se) ==
              length(thetahat), is.finite(se), min(se) > 0,
            !is.null(alternative))
  alternative <- match.arg(alternative)
  stopifnot(is.numeric(level), length(level) == 1, is.finite(level),
            0 < level, level < 1)
  target <- function(limit) {
    sqrt(pValueMu(thetahat = thetahat, se = se, c = c, mu = limit,
             alternative = alternative, bound = FALSE)) - alpha
  }
  indOrd <- order(thetahat)
  thetahat <- thetahat[indOrd]
  se <- se[indOrd]
  thetahatUnique <- unique(thetahat)
  nThetahatUnique <- length(thetahatUnique)
  mini <- which.min(thetahat)
  maxi <- which.max(thetahat)
  mint <- thetahat[mini]
  maxt <- thetahat[maxi]
  minse <- se[mini]
  maxse <- se[maxi]
  alpha <- 1 - level
  z1 <- max(-qnorm(alpha), 1)
  eps <- 1e-06
  factor <- 5
  if (alternative == "none") {
    lower <- mint - z1 * minse
    while (target(lower) > 0) lower <- lower - minse
    CIlower <- uniroot(f = target, lower = lower, upper = thetahat[1])$root
    CImiddle <- matrix(NA, nrow = 2, ncol = nThetahatUnique -
                         1)
    gam <- matrix(NA, nrow = nThetahatUnique - 1, ncol = 2)
    colnames(gam) <- c("minimum", "pvalue_fun/gamma")
    for (i in 1:(nThetahatUnique - 1)) {
      opt <- optimize(f = target, lower = thetahatUnique[i],
                      upper = thetahatUnique[i + 1])
      gam[i, ] <- c(opt$minimum, opt$objective + alpha)
      if (opt$objective <= 0) {
        CImiddle[1, i] <- uniroot(f = target, lower = thetahatUnique[i],
                                  upper = opt$minimum)$root
        CImiddle[2, i] <- uniroot(f = target, lower = opt$minimum,
                                  upper = thetahatUnique[i + 1])$root
      }
    }
    CImiddle <- CImiddle[!is.na(CImiddle)]
    upper <- maxt + maxse
    while (target(upper) > 0) upper <- upper + z1 * maxse
    CIupper <- uniroot(f = target, lower = thetahat[length(thetahat)],
                       upper = upper)$root
    CI <- matrix(c(CIlower, CImiddle, CIupper), ncol = 2,
                 byrow = TRUE)
    colnames(CI) <- c("lower", "upper")
    return(list(CI = CI, gamma = gam))
  }
  if (alternative == "two.sided") {
    lower <- uniroot(f = target, lower = mint - factor *
                       z1 * minse, upper = mint - eps * minse)$root
    upper <- uniroot(f = target, lower = maxt + eps * maxse,
                     upper = maxt + factor * z1 * maxse)$root
    return(list(CI = cbind(lower, upper)))
  }
  if (alternative == "greater") {
    lower <- uniroot(f = target, lower = mint - factor *
                       z1 * minse, upper = mint - eps * minse)$root
    upper <- Inf
    return(list(CI = cbind(lower, upper)))
  }
  if (alternative == "less") {
    lower <- -Inf
    upper <- uniroot(f = target, lower = maxt + eps * maxse,
                     upper = maxt + factor * z1 * maxse)$root
    return(list(CI = cbind(lower, upper)))
  }
  stop("function not get here.")
}


## doesn't work currently
## T1EpSceptical(alternative="less", level=0.025, c=0.85, type="golden")/2
## PPpSceptical(alternative="less", level=0.025, c=0.85, type="golden", alpha=0.025, power=0.8)

## function target compares T1E at a certain alphalevel to targetT1E for a given c
## used in alphaLevel
target <- function(alphalevel, alternative=alternative, c=c, targetT1E){
    myT1E <- T1EpSceptical(alternative=alternative, level=alphalevel, c=c, type="nominal")
    #### we may also use this:
    ## myT1E <- 1-FZ(p2z(alphalevel, alternative=alternative)^2, c=c)
    ## if(alternative=="one.sided") myT1E <- myT1E/2
    ####
    ## divide by two to 1-1 scenario
    result <- myT1E/2 - targetT1E
}

## computes the required alphalevel to achieve a certain targetT1E for a given c
alphaLevel <- function(c, alternative="one.sided", targetT1E){
    mylower <- sqrt(targetT1E)
    if(alternative=="one.sided")
        myupper <- 0.5
    if(alternative=="two.sided")
        myupper <- 1-.Machine$double.eps^0.25
    res <- uniroot(target, lower=mylower, upper=myupper,
                   alternative=alternative, c=c,
                   targetT1E=targetT1E)
    return(res$root)
}


ctarget <- function(c, targetT1E, level){
    term1 <- alphaLevel(c, alternative="one.sided", targetT1E=targetT1E)
    res <- term1 - level
    return(res)
}

## computes the value of c for a given alphalevel
cCalibrated <- function(targetT1E, alphalevel){
    eps <- 0.0001
    res <- uniroot(ctarget, lower=eps, upper=1/eps,
                   level=alphalevel, targetT1E=targetT1E)
    return(res$root)
}


pValueMu2 <- function(thetao, thetar, seo, ser, c, mu,
                         alternative = "none", bound = FALSE){
    pValueMu(thetahat=c(thetao, thetar), se=c(seo, ser),
             mu = mu, c = c,
             alternative = alternative,
             bound = bound)
}

PValueMu <- Vectorize(pValueMu)

ggPvalueFunction <- function(thetahat, se, level = 0.95,
                             xlim = c(min(0, thetahat - 2 * se),
                                      max(0, thetahat + 2 * se)),
                             c = NA,
                             squareRoot = FALSE){

    stopifnot(
        is.numeric(level),
        level > 0 && level < 1,
        c >= 0 || is.na(c)==TRUE,
        is.numeric(thetahat),
        length(thetahat) > 0L,
        is.numeric(se),
        length(se) == length(thetahat) || length(se) == 1L,
        is.numeric(xlim),
        length(xlim) == 2L,
        xlim[1] < xlim[2]
    )


  if(squareRoot == FALSE){
    alpha <- 1 - level
    mg <- metagen(TE = thetahat, seTE = se)
    eps <- 0.0025 # for plotting error bars
    eb_height <- 0.025
    if(is.na(c))
        c <- (se[1]/se[2])^2
    z1 <- thetahat[1]/se[1]
    z2 <- thetahat[2]/se[2]


    if(c == Inf)
        pS <- sqrt(pLimit(z1 = z1, z2 = z2))
    else
      pS <- pSceptical(zo = z1, zr = z2, c = c, alternative = "two.sided",
                       type = "controlled")

    muSeq <- seq(xlim[1], xlim[2], length.out = 1000)
    createData <- function(){
        pval <- numeric()
        for(i in 1:length(muSeq))
            pval[i] <- pValueMu(thetahat = thetahat, se = se,
                                c = c, mu = muSeq[i],
                                alternative = "none", bound = FALSE)

        CIs <- scepticalCI(thetahat = thetahat, se = se, c = c,
                        alternative = "none", level = level)
        idx <- which.min(CIs$gamma[,2])
        gamma_min <- CIs$gamma[idx,2]
        x_gamma_min <- CIs$gamma[idx,1]
        df1 <- data.frame(x = muSeq,
                          y = pval,
                          x_gamma = rep(x_gamma_min, length(muSeq)),
                          y_gamma = rep(gamma_min, length(muSeq)),
                                        #errorbar = err,
                          stringsAsFactors = FALSE)
        df2 <- data.frame(xmin = unname(CIs$CI[, 1]),
                          xmax = unname(CIs$CI[, 2]),
                          y = rep(alpha, nrow(CIs$CI)),
                          stringsAsFactors = FALSE)
        df2$ymax <- df2$y + eb_height
        df2$ymin <- df2$y - eb_height
        list(df1, df2)
    }
    data <-  createData()

    lines <- data[[1]] ##do.call(`rbind`, lapply(data, `[[`, i = 1L))
    errorbars <- data[[2]] ## do.call(`rbind`, lapply(data, `[[`, i = 2L))

    trans <- function(x) abs(x - 1) * 100
    breaks_y1 <- sort(c(alpha, pretty(lines$y)))
    breaks_y2 <- sort(trans(c(breaks_y1)))
    transparency <- 1
    Q <- Qtest(thetahat[1], thetahat[2], se[1], se[2])

    if(pS^2 >= 0.0001)
        title <- paste("min pH: p =", formatPval(lines$y_gamma[1]), "Q-test: p =", formatPval(Q),
                       "c =", format(c, digits=2, nsmall=2), "two-sided p =", formatPval(pS^2))
    else
    title <- paste("min pH: p =", formatPval(lines$y_gamma[1]), "Q-test: p =", formatPval(Q),
                       "c =", format(c, digits=2, nsmall=2), "two-sided p", formatPval(pS^2))

    ggplot(data = lines, aes(x = x, y = y)) +
        geom_line(alpha = transparency) +
        geom_point(aes(x = x_gamma, y = y_gamma), alpha = transparency) +
        geom_point(aes(x = 0, y = pS^2), alpha = transparency) +
        geom_hline(yintercept = alpha, linetype = "dashed") +
        geom_vline(xintercept = thetahat, linetype = "dashed", col=c(5,6)) +
        geom_vline(xintercept = 0, linetype = "dashed", col="darkgrey") +
        scale_y_continuous(name = "p-value",
                           breaks = breaks_y1,
                           limits = c(0, 1),
                           sec.axis = sec_axis(trans = trans,
                                               name = "Confidence level [%]",
                                               breaks = breaks_y2)) +
        geom_segment(data = errorbars, aes(x = xmin, xend = xmax, y = y, yend = y)) +
        geom_segment(data = errorbars, aes(x = xmin, xend = xmin, y = ymin, yend = ymax)) +
        geom_segment(data = errorbars, aes(x = xmax, xend = xmax, y = ymin, yend = ymax)) +
        xlim(xlim) +
        labs(x = bquote(mu),
             color = "Heterogeneity",
             title = title) +
        theme_minimal() +
        theme(axis.title.y.right = element_text(angle = 90),
              legend.position = "bottom")

  } else if(squareRoot == TRUE){
    ## new CM
    alpha <- sqrt(1 - level)
    mg <- metagen(TE = thetahat, seTE = se)
    eps <- 0.0025 # for plotting error bars
    eb_height <- 0.025
    if(is.na(c))
      c <- (se[1]/se[2])^2
    z1 <- thetahat[1]/se[1]
    z2 <- thetahat[2]/se[2]
    ylim = c(-0.05, 1.05)


    if(c == Inf)
      pS <- sqrt(pLimit(z1 = z1, z2 = z2))
    else
      pS <- pSceptical(zo = z1, zr = z2, c = c, alternative = "two.sided",
                       type = "controlled")

    muSeq <- seq(xlim[1], xlim[2], length.out = 1000)
    createData <- function(){
      pval <- numeric()
      for(i in 1:length(muSeq))
        pval[i] <- sqrt(pValueMu(thetahat = thetahat, se = se,
                            c = c, mu = muSeq[i],
                            alternative = "none", bound = FALSE))

      CIs <- scepticalCI2(thetahat = thetahat, se = se, c = c,
                      alternative = "none", level = (1 - alpha))
      idx <- which.min(CIs$gamma[,2])
      gamma_min <- CIs$gamma[idx,2]
      x_gamma_min <- CIs$gamma[idx,1]
      df1 <- data.frame(x = muSeq,
                        y = pval,
                        x_gamma = rep(x_gamma_min, length(muSeq)),
                        y_gamma = rep(gamma_min, length(muSeq)),
                        #errorbar = err,
                        stringsAsFactors = FALSE)
      df2 <- data.frame(xmin = unname(CIs$CI[, 1]),
                        xmax = unname(CIs$CI[, 2]),
                        y = rep(alpha, nrow(CIs$CI)),
                        stringsAsFactors = FALSE)
      df2$ymax <- df2$y + eb_height
      df2$ymin <- df2$y - eb_height
      list(df1, df2)
    }
    data <-  createData()

    lines <- data[[1]] ##do.call(`rbind`, lapply(data, `[[`, i = 1L))
    errorbars <- data[[2]] ## do.call(`rbind`, lapply(data, `[[`, i = 2L))

    trans <- function(x) (1 - x^2) * 100
    # breaks_y1 <- sort(c(alpha, pretty(lines$y)))
    breaks_y1 <- c(0, 0.2, 0.4, 0.6, 0.8,1)
    # breaks_y2 <- sort(trans(c(breaks_y1)))
    breaks_y2 <- trans(breaks_y1)
    transparency <- 1
    Q <- Qtest(thetahat[1], thetahat[2], se[1], se[2])

    if(pS >= 0.0001)
      title <- paste("min pH: p =", formatPval(lines$y_gamma[1]), "Q-test: p =", formatPval(Q),
                     "c =", format(c, digits=2, nsmall=2), "two-sided sceptical p =", formatPval(pS))
    else
      title <- paste("min pH: p =", formatPval(lines$y_gamma[1]), "Q-test: p =", formatPval(Q),
                     "c =", format(c, digits=2, nsmall=2), "two-sided sceptical p", formatPval(pS))

    ggplot(data = lines, aes(x = x, y = y)) +
      geom_line(alpha = transparency) +
      geom_point(aes(x = x_gamma, y = y_gamma), alpha = transparency) +
      geom_point(aes(x = 0, y = pS), alpha = transparency) +
      geom_hline(yintercept = alpha, linetype = "dashed") +
      geom_vline(xintercept = thetahat, linetype = "dashed", col=c(5,6)) +
      geom_vline(xintercept = 0, linetype = "dashed", col="darkgrey") +
      scale_y_continuous(name = "sceptical p-value",
                         breaks = breaks_y1,
                         limits = c(0, 1),
                         sec.axis = sec_axis(trans = trans,
                                             # breaks = trans(c(0, 0.2, 0.4, 0.6, 0.8, 1)),
                                              breaks = c(100, level*100, 84, 64, 36, 0),
                                             name = "Confidence level [%]")) +
                                             # breaks = breaks_y2)) +
      geom_segment(data = errorbars, aes(x = xmin, xend = xmax, y = y, yend = y)) +
      geom_segment(data = errorbars, aes(x = xmin, xend = xmin, y = ymin, yend = ymax)) +
      geom_segment(data = errorbars, aes(x = xmax, xend = xmax, y = ymin, yend = ymax)) +
      # addition Charlotte
    # geom_hline(yintercept = pS, linetype = "dashed", col = "red") +
      xlim(xlim) +
      labs(x = bquote(mu),
           color = "Heterogeneity",
           title = title) +
      theme_minimal() +
      theme(axis.title.y.right = element_text(angle = 90),
            # axis.text.y.right = element_text(size = 5),
            legend.position = "bottom")


  }
}

# define ForestPlot function (nearly unchanged from hMeanChiSqCI.Rnw) ----------------------------------------------------

ForestPlot <- function(thetahat, se,
                       barHeight = 0.5,
                       arrowHeight = 0.2,
                       lwd = 1.1,
                       cex = 2,
                       level = 0.95,
                       studyNames = c("original", "replication"),
                       TTR = FALSE,
                       title = NA){

  diamond <- function(center, height, width) {
    base <- matrix(c(1, 0, 0, 1, -1, 0, 0, -1), nrow = 2)
    trans <- rbind(base[1,] * width, base[2,] * height) + center
    geom_polygon(data = as.data.frame(t(trans)), mapping = aes(x = V1, y = V2))
  }



  #############################################
    if(TTR == FALSE)
        c <- (se[1]/se[2])^2
    if(TTR == TRUE)
        c <- 0
    meta <- metagen(TE = thetahat, seTE = se, random = TRUE, level.ci = level)
    CIs <- scepticalCI(thetahat = thetahat, se = se, c = c,
                        alternative = "none", level = level)

##    if(is.na(studyNames))
##        studies <- tibble(y = thetahat,
##                          lower = thetahat - 1.96 * se,
##                          upper = thetahat + 1.96 * se,
##                          names = paste("study", seq_along(y)))
##    else
        studies <- tibble(y = thetahat,
                          lower = thetahat - 1.96 * se,
                          upper = thetahat + 1.96 * se,
                          names = studyNames)


  hMean <- tibble(lower = CIs$CI[,1],
                  upper = CIs$CI[,2])

  fixedEffect <- tibble(lower = meta$lower.fixed,
                         upper = meta$upper.fixed)
  randomEffect <- tibble(lower = meta$lower.random,
                         upper = meta$upper.random)

    mmin <- min(studies$lower, fixedEffect$lower, hMean$lower)
    mmax <- max(studies$upper, fixedEffect$upper, hMean$upper)
    plotMargin <- mmin - 2.125 * (mmax - mmin)/3
    textStart <- mmin - 2 * (mmax - mmin)/3

  p <- ggplot(data = studies,
              mapping = aes(x=y, y=rev(seq_along(y)))) +
    geom_errorbar(width = barHeight, xmin = studies$lower, xmax = studies$upper, size=lwd, col=c(5,6)) +
    geom_point(shape = 15, col=c(5,6)) +
    geom_vline(xintercept = 0, linetype = "dashed", col="darkgrey") +
    xlim(c(plotMargin, mmax)) +
    geom_text(aes(label = names), x = textStart, hjust = 0) +
    xlab(expression(mu)) +
    scale_y_continuous(expand = c(.1, .1)) +
    theme_classic() +
    theme(axis.title.y=element_blank(),
          axis.text.y=element_blank(),
          axis.ticks.y=element_blank(),
          axis.line.y=element_blank(),
          plot.title = element_text(size = 30, face = "bold", hjust=0.5)) +
      ggtitle(title)

  if(!is.null(hMean)){
    p <- p + geom_segment(data = hMean,
                          mapping = aes(x = lower, xend = upper),
                          y= 0, yend=0,
                          arrow = arrow(ends="both", length = unit(arrowHeight, "in")),
                          color="red", size=lwd)
    p <- p + geom_text(label = "sceptical", x = textStart, y = 0, hjust = 0) +
      geom_point(data = studies, mapping = aes(x=y), y=0, color="blue", size=cex)
  }

  if(!is.null(fixedEffect)){
    p <- p + diamond(center = c(mean(c(fixedEffect$lower, fixedEffect$upper)), -1),
                     height = .2, width = (fixedEffect$upper-fixedEffect$lower)/2)
    p <- p + geom_text(label = "meta-analysis", x = textStart, y = -1 , hjust = 0)
  }
  ## if(!is.null(randomEffect)){
  ##   p <- p + diamond(center = c(mean(c(randomEffect$lower, randomEffect$upper)), -1),
  ##                    height = .2, width = (randomEffect$upper-randomEffect$lower)/2)
  ##   p <- p + geom_text(label = "random effects", x = textStart, y = -1 , hjust = 0)
  ## }
    return(p)
}
